@@ -1,12 +1,20 @@
 """ The basic field attributes. """
 
 import warnings
-from pymongo.dbref import DBRef
+
+# PyMongo moving libraries around -- eventually this can
+# go away.
+try:
+    from pymongo.dbref import DBRef
+except ImportError:
+    from bson.dbref import DBRef
+
 
 class EmptyRequiredField(Exception):
     """ Raised when a required field is not set on the model instance. """
     pass
 
+
 class Field(object):
     """
     This class may eventually do type-checking, default values,
@@ -47,32 +55,28 @@ class Field(object):
         value = None
         if not field_name in instance:
             if self.required:
-                raise EmptyRequiredField("'%s' is required but is empty."
-                                         % field_name)
-            else:
-                value = self._get_default()
-                if hasattr(self, "default"):
-                    # setting the default value on the instance
-                    instance[field_name] = value
-        else:
-            # value already set on the dictionary instance
-            value = instance[field_name]
+                raise EmptyRequiredField(
+                    "'%s' is required but is empty." % field_name)
+            self._set_default(instance, field_name)
+        value = instance.get(field_name)
         if self._get_callback:
             value = self._get_callback(instance, value)
         return value
 
-    def _get_default(self):
-        """ Retrieve the default value and return it """
+    def _set_default(self, model, field):
+        if field in model:
+            # value already set, not overwriting it.
+            return
         if hasattr(self, "default"):
-            if callable(self.default):
-                return self.default()
+            if not callable(self.default):
+                warnings.warn(
+                    "Defaults that are not functions (or callable) "
+                    "are deprecated, and will be removed in a future version.",
+                    DeprecationWarning)
+                default_value = self.default
             else:
-                return self.default
-        else:
-            warnings.warn("Field has no value AND no default value -- "
-                "in a future release, this will raise an exception.",
-                DeprecationWarning)
-            return None
+                default_value = self.default()
+            setattr(model, field, default_value)
 
     def _check_value_type(self, value):
         """ Verifies that a value is the proper type """
@@ -85,9 +89,8 @@ class Field(object):
     def __set__(self, instance, value):
         value_type = type(value)
         if not self._check_value_type(value):
-            raise TypeError("Invalid type %s instead of %s" %
-                (value_type, self.value_type)
-            )
+            raise TypeError("Invalid type %s instead of %s" % (
+                value_type, self.value_type))
         if self._set_callback:
             value = self._set_callback(instance, value)
         field_name = self._get_field_name(instance)
@@ -114,6 +117,7 @@ class ReferenceField(Field):
             return self.model.find_one({"_id": value.id})
         return value
 
+
 class ConstantField(Field):
     """ Doesn't let you change the value after setting it. """
 
